### R code from vignette source 'CAPM_TF.Rnw'
### Encoding: ISO8859-1

###################################################
### code chunk number 1: ex1
###################################################
# 'Load the GARPFRM package and CRSP dataset for CAPM analysis.
suppressMessages(library(GARPFRM))
options(digits=3)
data(crsp.short)

stock.df <- largecap.ts[, 1:20]
mrkt <- largecap.ts[, "market"]
rfr <- largecap.ts[, "t90"]

# Plot first four stocks from 
plot.zoo(stock.df[,1:4], main="First Four Large Cap Returns")


###################################################
### code chunk number 2: ex2
###################################################
# Illustrate the type of data being analzyed: start-end dates.
start(stock.df[,1:4])
end(stock.df[,1:4])
# Count the number of rows: sample size.
nrow(stock.df)


###################################################
### code chunk number 3: ex3
###################################################
# Excess Returns initialized before utilizing in CAPM
exReturns <- Return.excess(stock.df, rfr)
colnames(exReturns)= c(colnames(stock.df))


###################################################
### code chunk number 4: ex4
###################################################
# Univariate CAPM
uv <- CAPM(exReturns[,1], mrkt)
getStatistics(uv)

# Plot data with regression line
plot(uv)


###################################################
### code chunk number 5: ex5
###################################################
# MLM CAPM for AMAT, AMGN, and CAT
mlm <- CAPM(exReturns[,1:3], mrkt)
getStatistics(mlm)

# Plot data with regression line
plot(mlm)


###################################################
### code chunk number 6: ex6
###################################################
# For uv
getBetas(uv)
getAlphas(uv)
hypTest(uv, significanceLevel=0.05)
# For mlm
getBetas(mlm)
getAlphas(mlm)
hypTest(mlm, significanceLevel=0.05)


###################################################
### code chunk number 7: ex7
###################################################
# MLM CAPM
mlm <- CAPM(exReturns[,], mrkt)

# Plot expected returns versus betas
chartSML(mlm)


###################################################
### code chunk number 8: ex8
###################################################
# Load FED consumption data: CONS
data(consumption)

# Convert to yearmon index and align consumption and mrkt
consumption <- xts(consumption, as.yearmon(index(consumption)))
mrkt <- xts(mrkt, as.yearmon(index(mrkt)))
consumption <- consumption[index(mrkt)]

capm.cons = CAPM(consumption, mrkt)
coef(summary(capm.cons))

# Plot data with regression line
plot(capm.cons)


